@@ -49,27 +49,22 @@ struct _NautilusBatchRenameDialog
     GtkWidget *replace_entry;
     GtkWidget *format_mode_button;
     GtkWidget *replace_mode_button;
-    GtkWidget *add_button;
-    GtkWidget *add_popover;
     GtkWidget *numbering_order_label;
     GtkWidget *numbering_label;
     GtkWidget *scrolled_window;
-    GtkWidget *numbering_order_popover;
-    GtkWidget *numbering_order_button;
     GtkWidget *numbering_revealer;
     GtkWidget *conflict_box;
     GtkWidget *conflict_label;
     GtkWidget *conflict_down;
     GtkWidget *conflict_up;
 
-    GList *original_name_listbox_rows;
-    GList *arrow_listbox_rows;
-    GList *result_listbox_rows;
     GList *listbox_labels_new;
     GList *listbox_labels_old;
     GList *listbox_icons;
     GtkSizeGroup *size_group;
 
+    GList *motion_controllers;
+
     GList *selection;
     GList *new_names;
     NautilusBatchRenameDialogMode mode;
@@ -78,7 +73,6 @@ struct _NautilusBatchRenameDialog
     GActionGroup *action_group;
 
     GMenu *numbering_order_menu;
-    GMenu *add_tag_menu;
 
     GHashTable *create_date;
     GList *selection_metadata;
@@ -150,8 +144,6 @@ change_numbering_order (GSimpleAction *action,
 
     g_simple_action_set_state (action, value);
 
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->numbering_order_button), FALSE);
-
     update_display_text (dialog);
 }
 
@@ -490,15 +482,18 @@ listbox_header_func (GtkListBoxRow             *row,
                      GtkListBoxRow             *before,
                      NautilusBatchRenameDialog *dialog)
 {
-    gboolean show_separator;
+    GtkWidget *separator;
 
-    show_separator = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row),
-                                                         "show-separator"));
-
-    if (show_separator)
+    if (before == NULL)
     {
-        GtkWidget *separator;
+        /* First row needs no separator */
+        gtk_list_box_row_set_header (row, NULL);
+        return;
+    }
 
+    separator = gtk_list_box_row_get_header (row);
+    if (separator == NULL)
+    {
         separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
         gtk_widget_show (separator);
 
@@ -512,7 +507,7 @@ static void
 update_rows_height (NautilusBatchRenameDialog *dialog)
 {
     GList *l;
-    gint current_row_natural_height;
+    GtkRequisition current_row_natural_size;
     gint maximum_height;
 
     maximum_height = -1;
@@ -520,37 +515,37 @@ update_rows_height (NautilusBatchRenameDialog *dialog)
     /* check if maximum height has changed */
     for (l = dialog->listbox_labels_new; l != NULL; l = l->next)
     {
-        gtk_widget_get_preferred_height (GTK_WIDGET (l->data),
-                                         NULL,
-                                         &current_row_natural_height);
+        gtk_widget_get_preferred_size (GTK_WIDGET (l->data),
+                                       NULL,
+                                       &current_row_natural_size);
 
-        if (current_row_natural_height > maximum_height)
+        if (current_row_natural_size.height > maximum_height)
         {
-            maximum_height = current_row_natural_height;
+            maximum_height = current_row_natural_size.height;
         }
     }
 
     for (l = dialog->listbox_labels_old; l != NULL; l = l->next)
     {
-        gtk_widget_get_preferred_height (GTK_WIDGET (l->data),
-                                         NULL,
-                                         &current_row_natural_height);
+        gtk_widget_get_preferred_size (GTK_WIDGET (l->data),
+                                       NULL,
+                                       &current_row_natural_size);
 
-        if (current_row_natural_height > maximum_height)
+        if (current_row_natural_size.height > maximum_height)
         {
-            maximum_height = current_row_natural_height;
+            maximum_height = current_row_natural_size.height;
         }
     }
 
     for (l = dialog->listbox_icons; l != NULL; l = l->next)
     {
-        gtk_widget_get_preferred_height (GTK_WIDGET (l->data),
-                                         NULL,
-                                         &current_row_natural_height);
+        gtk_widget_get_preferred_size (GTK_WIDGET (l->data),
+                                       NULL,
+                                       &current_row_natural_size);
 
-        if (current_row_natural_height > maximum_height)
+        if (current_row_natural_size.height > maximum_height)
         {
-            maximum_height = current_row_natural_height;
+            maximum_height = current_row_natural_size.height;
         }
     }
 
@@ -576,17 +571,11 @@ update_rows_height (NautilusBatchRenameDialog *dialog)
 }
 
 static GtkWidget *
-create_original_name_row_for_label (NautilusBatchRenameDialog *dialog,
-                                    const gchar               *old_text,
-                                    gboolean                   show_separator)
+create_original_name_label (NautilusBatchRenameDialog *dialog,
+                            const gchar               *old_text)
 {
-    GtkWidget *row;
     GtkWidget *label_old;
 
-    row = gtk_list_box_row_new ();
-
-    g_object_set_data (G_OBJECT (row), "show-separator", GINT_TO_POINTER (show_separator));
-
     label_old = gtk_label_new (old_text);
     gtk_label_set_xalign (GTK_LABEL (label_old), 0.0);
     gtk_widget_set_hexpand (label_old, TRUE);
@@ -595,24 +584,17 @@ create_original_name_row_for_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_labels_old = g_list_prepend (dialog->listbox_labels_old, label_old);
 
-    gtk_container_add (GTK_CONTAINER (row), label_old);
-    gtk_widget_show_all (row);
+    gtk_widget_show_all (label_old);
 
-    return row;
+    return label_old;
 }
 
 static GtkWidget *
-create_result_row_for_label (NautilusBatchRenameDialog *dialog,
-                             const gchar               *new_text,
-                             gboolean                   show_separator)
+create_result_label (NautilusBatchRenameDialog *dialog,
+                     const gchar               *new_text)
 {
-    GtkWidget *row;
     GtkWidget *label_new;
 
-    row = gtk_list_box_row_new ();
-
-    g_object_set_data (G_OBJECT (row), "show-separator", GINT_TO_POINTER (show_separator));
-
     label_new = gtk_label_new (new_text);
     gtk_label_set_xalign (GTK_LABEL (label_new), 0.0);
     gtk_widget_set_hexpand (label_new, TRUE);
@@ -621,24 +603,18 @@ create_result_row_for_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_labels_new = g_list_prepend (dialog->listbox_labels_new, label_new);
 
-    gtk_container_add (GTK_CONTAINER (row), label_new);
-    gtk_widget_show_all (row);
+    gtk_widget_show_all (label_new);
 
-    return row;
+    return label_new;
 }
 
 static GtkWidget *
-create_arrow_row_for_label (NautilusBatchRenameDialog *dialog,
-                            gboolean                   show_separator)
+create_arrow (NautilusBatchRenameDialog *dialog,
+              GtkTextDirection           text_direction)
 {
-    GtkWidget *row;
     GtkWidget *icon;
 
-    row = gtk_list_box_row_new ();
-
-    g_object_set_data (G_OBJECT (row), "show-separator", GINT_TO_POINTER (show_separator));
-
-    if (gtk_widget_get_direction (row) == GTK_TEXT_DIR_RTL)
+    if (text_direction == GTK_TEXT_DIR_RTL)
     {
         icon = gtk_label_new ("←");
     }
@@ -653,10 +629,9 @@ create_arrow_row_for_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_icons = g_list_prepend (dialog->listbox_icons, icon);
 
-    gtk_container_add (GTK_CONTAINER (row), icon);
-    gtk_widget_show_all (row);
+    gtk_widget_show_all (icon);
 
-    return row;
+    return icon;
 }
 
 static void
@@ -719,47 +694,37 @@ batch_rename_dialog_on_response (NautilusBatchRenameDialog *dialog,
 static void
 fill_display_listbox (NautilusBatchRenameDialog *dialog)
 {
-    GtkWidget *row;
+    GtkWidget *row_child;
     GList *l1;
     GList *l2;
     NautilusFile *file;
     GString *new_name;
     gchar *name;
-
-    dialog->original_name_listbox_rows = NULL;
-    dialog->arrow_listbox_rows = NULL;
-    dialog->result_listbox_rows = NULL;
+    GtkTextDirection text_direction;
 
     gtk_size_group_add_widget (dialog->size_group, dialog->result_listbox);
     gtk_size_group_add_widget (dialog->size_group, dialog->original_name_listbox);
 
+    text_direction = gtk_widget_get_direction (GTK_WIDGET (dialog));
+
     for (l1 = dialog->new_names, l2 = dialog->selection; l1 != NULL && l2 != NULL; l1 = l1->next, l2 = l2->next)
     {
         file = NAUTILUS_FILE (l2->data);
         new_name = l1->data;
 
         name = nautilus_file_get_name (file);
-        row = create_original_name_row_for_label (dialog, name, TRUE);
-        gtk_container_add (GTK_CONTAINER (dialog->original_name_listbox), row);
-        dialog->original_name_listbox_rows = g_list_prepend (dialog->original_name_listbox_rows,
-                                                             row);
+        row_child = create_original_name_label (dialog, name);
+        gtk_list_box_insert (GTK_LIST_BOX (dialog->original_name_listbox), row_child, -1);
 
-        row = create_arrow_row_for_label (dialog, TRUE);
-        gtk_container_add (GTK_CONTAINER (dialog->arrow_listbox), row);
-        dialog->arrow_listbox_rows = g_list_prepend (dialog->arrow_listbox_rows,
-                                                     row);
+        row_child = create_arrow (dialog, text_direction);
+        gtk_list_box_insert (GTK_LIST_BOX (dialog->arrow_listbox), row_child, -1);
 
-        row = create_result_row_for_label (dialog, new_name->str, TRUE);
-        gtk_container_add (GTK_CONTAINER (dialog->result_listbox), row);
-        dialog->result_listbox_rows = g_list_prepend (dialog->result_listbox_rows,
-                                                      row);
+        row_child = create_result_label (dialog, new_name->str);
+        gtk_list_box_insert (GTK_LIST_BOX (dialog->result_listbox), row_child, -1);
 
         g_free (name);
     }
 
-    dialog->original_name_listbox_rows = g_list_reverse (dialog->original_name_listbox_rows);
-    dialog->arrow_listbox_rows = g_list_reverse (dialog->arrow_listbox_rows);
-    dialog->result_listbox_rows = g_list_reverse (dialog->result_listbox_rows);
     dialog->listbox_labels_old = g_list_reverse (dialog->listbox_labels_old);
     dialog->listbox_labels_new = g_list_reverse (dialog->listbox_labels_new);
     dialog->listbox_icons = g_list_reverse (dialog->listbox_icons);
@@ -778,6 +743,7 @@ select_nth_conflict (NautilusBatchRenameDialog *dialog)
     GtkAdjustment *adjustment;
     GtkAllocation allocation;
     ConflictData *conflict_data;
+    GtkListBoxRow *list_box_row;
 
     nth_conflict = dialog->selected_conflict;
     l = g_list_nth (dialog->duplicates, nth_conflict);
@@ -789,17 +755,20 @@ select_nth_conflict (NautilusBatchRenameDialog *dialog)
 
     nth_conflict_index = conflict_data->index;
 
-    l = g_list_nth (dialog->original_name_listbox_rows, nth_conflict_index);
+    l = g_list_nth (dialog->listbox_labels_new, nth_conflict_index);
+    list_box_row = GTK_LIST_BOX_ROW (gtk_widget_get_parent (l->data));
     gtk_list_box_select_row (GTK_LIST_BOX (dialog->original_name_listbox),
-                             l->data);
+                             list_box_row);
 
-    l = g_list_nth (dialog->arrow_listbox_rows, nth_conflict_index);
+    l = g_list_nth (dialog->listbox_labels_old, nth_conflict_index);
+    list_box_row = GTK_LIST_BOX_ROW (gtk_widget_get_parent (l->data));
     gtk_list_box_select_row (GTK_LIST_BOX (dialog->arrow_listbox),
-                             l->data);
+                             list_box_row);
 
-    l = g_list_nth (dialog->result_listbox_rows, nth_conflict_index);
+    l = g_list_nth (dialog->listbox_icons, nth_conflict_index);
+    list_box_row = GTK_LIST_BOX_ROW (gtk_widget_get_parent (l->data));
     gtk_list_box_select_row (GTK_LIST_BOX (dialog->result_listbox),
-                             l->data);
+                             list_box_row);
 
     /* scroll to the selected row */
     adjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (dialog->scrolled_window));
@@ -886,22 +855,26 @@ update_conflict_row_background (NautilusBatchRenameDialog *dialog)
 
     duplicates = dialog->duplicates;
 
-    for (l1 = dialog->original_name_listbox_rows,
-         l2 = dialog->arrow_listbox_rows,
-         l3 = dialog->result_listbox_rows;
+    for (l1 = dialog->listbox_labels_new,
+         l2 = dialog->listbox_labels_old,
+         l3 = dialog->listbox_icons;
          l1 != NULL && l2 != NULL && l3 != NULL;
          l1 = l1->next, l2 = l2->next, l3 = l3->next)
     {
-        context = gtk_widget_get_style_context (GTK_WIDGET (l1->data));
+        GtkWidget *row1 = gtk_widget_get_parent (l1->data);
+        GtkWidget *row2 = gtk_widget_get_parent (l2->data);
+        GtkWidget *row3 = gtk_widget_get_parent (l3->data);
+
+        context = gtk_widget_get_style_context (row1);
 
         if (gtk_style_context_has_class (context, "conflict-row"))
         {
             gtk_style_context_remove_class (context, "conflict-row");
 
-            context = gtk_widget_get_style_context (GTK_WIDGET (l2->data));
+            context = gtk_widget_get_style_context (row2);
             gtk_style_context_remove_class (context, "conflict-row");
 
-            context = gtk_widget_get_style_context (GTK_WIDGET (l3->data));
+            context = gtk_widget_get_style_context (row3);
             gtk_style_context_remove_class (context, "conflict-row");
         }
 
@@ -910,13 +883,13 @@ update_conflict_row_background (NautilusBatchRenameDialog *dialog)
             conflict_data = duplicates->data;
             if (conflict_data->index == index)
             {
-                context = gtk_widget_get_style_context (GTK_WIDGET (l1->data));
+                context = gtk_widget_get_style_context (row1);
                 gtk_style_context_add_class (context, "conflict-row");
 
-                context = gtk_widget_get_style_context (GTK_WIDGET (l2->data));
+                context = gtk_widget_get_style_context (row2);
                 gtk_style_context_add_class (context, "conflict-row");
 
-                context = gtk_widget_get_style_context (GTK_WIDGET (l3->data));
+                context = gtk_widget_get_style_context (row3);
                 gtk_style_context_add_class (context, "conflict-row");
 
                 duplicates = duplicates->next;
@@ -1433,44 +1406,6 @@ batch_rename_dialog_mode_changed (NautilusBatchRenameDialog *dialog)
     update_display_text (dialog);
 }
 
-static void
-add_button_clicked (NautilusBatchRenameDialog *dialog)
-{
-    if (gtk_widget_is_visible (dialog->add_popover))
-    {
-        gtk_widget_set_visible (dialog->add_popover, FALSE);
-    }
-    else
-    {
-        gtk_widget_set_visible (dialog->add_popover, TRUE);
-    }
-}
-
-static void
-add_popover_closed (NautilusBatchRenameDialog *dialog)
-{
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->add_button), FALSE);
-}
-
-static void
-numbering_order_button_clicked (NautilusBatchRenameDialog *dialog)
-{
-    if (gtk_widget_is_visible (dialog->numbering_order_popover))
-    {
-        gtk_widget_set_visible (dialog->numbering_order_popover, FALSE);
-    }
-    else
-    {
-        gtk_widget_set_visible (dialog->numbering_order_popover, TRUE);
-    }
-}
-
-static void
-numbering_order_popover_closed (NautilusBatchRenameDialog *dialog)
-{
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->numbering_order_button), FALSE);
-}
-
 void
 nautilus_batch_rename_dialog_query_finished (NautilusBatchRenameDialog *dialog,
                                              GHashTable                *hash_table,
@@ -1555,25 +1490,27 @@ update_row_shadowing (GtkWidget *row,
 
     if (shown)
     {
-        flags |= GTK_STATE_PRELIGHT;
+        flags |= GTK_STATE_FLAG_PRELIGHT;
     }
     else
     {
-        flags &= ~GTK_STATE_PRELIGHT;
+        flags &= ~GTK_STATE_FLAG_PRELIGHT;
     }
 
     gtk_style_context_set_state (context, flags);
 }
 
-static gboolean
-on_motion_notify (GtkWidget *widget,
-                  GdkEvent  *event,
-                  gpointer   user_data)
+static void
+on_event_controller_motion_motion (GtkEventControllerMotion *controller,
+                                   double                    x,
+                                   double                    y,
+                                   gpointer                  user_data)
 {
+    GtkWidget *widget;
     NautilusBatchRenameDialog *dialog;
-    gdouble y;
     GtkListBoxRow *row;
 
+    widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (controller));
     dialog = NAUTILUS_BATCH_RENAME_DIALOG (user_data);
 
     if (dialog->preselected_row1 && dialog->preselected_row2)
@@ -1582,11 +1519,6 @@ on_motion_notify (GtkWidget *widget,
         update_row_shadowing (dialog->preselected_row2, FALSE);
     }
 
-    if (G_UNLIKELY (!gdk_event_get_coords (event, NULL, &y)))
-    {
-        g_return_val_if_reached (GDK_EVENT_PROPAGATE);
-    }
-
     if (widget == dialog->result_listbox)
     {
         row = gtk_list_box_get_row_at_y (GTK_LIST_BOX (dialog->original_name_listbox), y);
@@ -1619,14 +1551,11 @@ on_motion_notify (GtkWidget *widget,
         update_row_shadowing (GTK_WIDGET (row), TRUE);
         dialog->preselected_row2 = GTK_WIDGET (row);
     }
-
-    return GDK_EVENT_PROPAGATE;
 }
 
-static gboolean
-on_leave_notify (GtkWidget *widget,
-                 GdkEvent  *event,
-                 gpointer   user_data)
+static void
+on_event_controller_motion_leave (GtkEventControllerMotion *controller,
+                                  gpointer                  user_data)
 {
     NautilusBatchRenameDialog *dialog;
 
@@ -1637,30 +1566,6 @@ on_leave_notify (GtkWidget *widget,
 
     dialog->preselected_row1 = NULL;
     dialog->preselected_row2 = NULL;
-
-    return GDK_EVENT_PROPAGATE;
-}
-
-static gboolean
-on_event (GtkWidget *widget,
-          GdkEvent  *event,
-          gpointer   user_data)
-{
-    GdkEventType event_type;
-
-    event_type = gdk_event_get_event_type (event);
-
-    if (event_type == GDK_MOTION_NOTIFY)
-    {
-        return on_motion_notify (widget, event, user_data);
-    }
-
-    if (event_type == GDK_LEAVE_NOTIFY)
-    {
-        return on_leave_notify (widget, event, user_data);
-    }
-
-    return GDK_EVENT_PROPAGATE;
 }
 
 static void
@@ -1920,11 +1825,6 @@ nautilus_batch_rename_dialog_finalize (GObject *object)
         cancel_conflict_check (dialog);
     }
 
-    g_clear_object (&dialog->numbering_order_menu);
-    g_clear_object (&dialog->add_tag_menu);
-    g_list_free (dialog->original_name_listbox_rows);
-    g_list_free (dialog->arrow_listbox_rows);
-    g_list_free (dialog->result_listbox_rows);
     g_list_free (dialog->listbox_labels_new);
     g_list_free (dialog->listbox_labels_old);
     g_list_free (dialog->listbox_icons);
@@ -1959,6 +1859,7 @@ nautilus_batch_rename_dialog_finalize (GObject *object)
     nautilus_directory_list_free (dialog->distinct_parent_directories);
 
     g_object_unref (dialog->size_group);
+    g_clear_list (&dialog->motion_controllers, g_object_unref);
 
     g_hash_table_destroy (dialog->tag_info_table);
 
@@ -1993,12 +1894,9 @@ nautilus_batch_rename_dialog_class_init (NautilusBatchRenameDialogClass *klass)
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, mode_stack);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, replace_mode_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, format_mode_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, add_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, add_popover);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_label);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, scrolled_window);
-    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_popover);
-    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_button);
+    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_menu);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_revealer);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, conflict_box);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, conflict_label);
@@ -2009,10 +1907,6 @@ nautilus_batch_rename_dialog_class_init (NautilusBatchRenameDialogClass *klass)
     gtk_widget_class_bind_template_callback (widget_class, file_names_widget_on_activate);
     gtk_widget_class_bind_template_callback (widget_class, file_names_widget_entry_on_changed);
     gtk_widget_class_bind_template_callback (widget_class, batch_rename_dialog_mode_changed);
-    gtk_widget_class_bind_template_callback (widget_class, add_button_clicked);
-    gtk_widget_class_bind_template_callback (widget_class, add_popover_closed);
-    gtk_widget_class_bind_template_callback (widget_class, numbering_order_button_clicked);
-    gtk_widget_class_bind_template_callback (widget_class, numbering_order_popover_closed);
     gtk_widget_class_bind_template_callback (widget_class, select_next_conflict_up);
     gtk_widget_class_bind_template_callback (widget_class, select_next_conflict_down);
     gtk_widget_class_bind_template_callback (widget_class, batch_rename_dialog_on_response);
@@ -2107,12 +2001,28 @@ nautilus_batch_rename_dialog_new (GList             *selection,
     return GTK_WIDGET (dialog);
 }
 
+static void
+connect_to_pointer_motion_events (NautilusBatchRenameDialog *self,
+                                  GtkWidget                 *listbox)
+{
+    GtkEventController *controller;
+
+    controller = gtk_event_controller_motion_new (listbox);
+    gtk_event_controller_set_propagation_phase (controller, GTK_PHASE_CAPTURE);
+    g_signal_connect (controller, "leave",
+                      G_CALLBACK (on_event_controller_motion_leave), self);
+    g_signal_connect (controller, "motion",
+                      G_CALLBACK (on_event_controller_motion_motion), self);
+
+    self->motion_controllers = g_list_prepend (self->motion_controllers,
+                                               controller);
+}
+
 static void
 nautilus_batch_rename_dialog_init (NautilusBatchRenameDialog *self)
 {
     TagData *tag_data;
     guint i;
-    g_autoptr (GtkBuilder) builder = NULL;
 
     gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -2132,17 +2042,6 @@ nautilus_batch_rename_dialog_init (NautilusBatchRenameDialog *self)
 
     self->mode = NAUTILUS_BATCH_RENAME_DIALOG_FORMAT;
 
-    builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/ui/nautilus-batch-rename-dialog-menu.ui");
-    self->numbering_order_menu = g_object_ref_sink (G_MENU (gtk_builder_get_object (builder, "numbering_order_menu")));
-    self->add_tag_menu = g_object_ref_sink (G_MENU (gtk_builder_get_object (builder, "add_tag_menu")));
-
-    gtk_popover_bind_model (GTK_POPOVER (self->numbering_order_popover),
-                            G_MENU_MODEL (self->numbering_order_menu),
-                            NULL);
-    gtk_popover_bind_model (GTK_POPOVER (self->add_popover),
-                            G_MENU_MODEL (self->add_tag_menu),
-                            NULL);
-
     gtk_label_set_ellipsize (GTK_LABEL (self->conflict_label), PANGO_ELLIPSIZE_END);
     gtk_label_set_max_width_chars (GTK_LABEL (self->conflict_label), 1);
 
@@ -2193,18 +2092,9 @@ nautilus_batch_rename_dialog_init (NautilusBatchRenameDialog *self)
 
     self->size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
-    g_signal_connect (self->original_name_listbox,
-                      "event",
-                      G_CALLBACK (on_event),
-                      self);
-    g_signal_connect (self->result_listbox,
-                      "event",
-                      G_CALLBACK (on_event),
-                      self);
-    g_signal_connect (self->arrow_listbox,
-                      "event",
-                      G_CALLBACK (on_event),
-                      self);
+    connect_to_pointer_motion_events (self, self->original_name_listbox);
+    connect_to_pointer_motion_events (self, self->result_listbox);
+    connect_to_pointer_motion_events (self, self->arrow_listbox);
 
     self->metadata_cancellable = g_cancellable_new ();
 }
