@@ -20,8 +20,7 @@
    Author: Ettore Perazzoli <ettore@gnu.org>
 */
 
-#ifndef NAUTILUS_CANVAS_CONTAINER_PRIVATE_H
-#define NAUTILUS_CANVAS_CONTAINER_PRIVATE_H
+#pragma once
 
 #include <eel/eel-glib-extensions.h>
 #include "nautilus-canvas-item.h"
@@ -193,15 +192,14 @@ struct NautilusCanvasContainerDetails {
 
 	int size_allocation_count;
 	guint size_allocation_count_id;
-	
-	/* Ignore the visible area the next time the scroll region is recomputed */
-	gboolean reset_scroll_region_trigger;
 
 	/* a11y items used by canvas items */
 	guint a11y_item_action_idle_handler;
 	GQueue* a11y_item_action_queue;
 
+	eel_boolean_bit in_layout_now : 1;
 	eel_boolean_bit is_loading : 1;
+	eel_boolean_bit is_populating_container : 1;
 	eel_boolean_bit needs_resort : 1;
 	eel_boolean_bit selection_needs_resort : 1;
 };
@@ -223,5 +221,3 @@ gboolean      nautilus_canvas_container_scroll                      (NautilusCan
 								     int                    delta_x,
 								     int                    delta_y);
 void          nautilus_canvas_container_update_scroll_region        (NautilusCanvasContainer *container);
-
-#endif /* NAUTILUS_CANVAS_CONTAINER_PRIVATE_H */
