@@ -44,6 +44,7 @@
 #include "nautilus-canvas-dnd.h"
 #include "nautilus-metadata.h"
 #include "nautilus-clipboard.h"
+#include "nautilus-gtk4-helpers.h"
 
 #define DEBUG_FLAG NAUTILUS_DEBUG_CANVAS_VIEW
 #include "nautilus-debug.h"
@@ -148,6 +149,18 @@ static const SortCriterion sort_criteria[] =
         "access-date-desc",
         TRUE
     },
+    {
+        NAUTILUS_FILE_SORT_BY_BTIME,
+        "creation date",
+        "creation-date",
+        FALSE
+    },
+    {
+        NAUTILUS_FILE_SORT_BY_BTIME,
+        "creation date",
+        "creation-date-desc",
+        TRUE
+    },
     {
         NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
         "trashed",
@@ -528,9 +541,9 @@ nautilus_canvas_view_begin_loading (NautilusFilesView *view)
 }
 
 static void
-on_clipboard_contents_received (GtkClipboard *clipboard,
-                                const gchar  *selection_data,
-                                gpointer      user_data)
+on_clipboard_contents_received (GtkClipboard     *clipboard,
+                                GtkSelectionData *selection_data,
+                                gpointer          user_data)
 {
     NautilusCanvasView *canvas_view;
 
@@ -569,9 +582,10 @@ static void
 update_clipboard_status (NautilusCanvasView *view)
 {
     g_object_ref (view);     /* Need to keep the object alive until we get the reply */
-    gtk_clipboard_request_text (nautilus_clipboard_get (GTK_WIDGET (view)),
-                                on_clipboard_contents_received,
-                                view);
+    gtk_clipboard_request_contents (nautilus_clipboard_get (GTK_WIDGET (view)),
+                                    nautilus_clipboard_get_atom (),
+                                    on_clipboard_contents_received,
+                                    view);
 }
 
 static void
@@ -1025,19 +1039,6 @@ canvas_container_activate_callback (NautilusCanvasContainer *container,
                                         0, TRUE);
 }
 
-static void
-canvas_container_activate_previewer_callback (NautilusCanvasContainer *container,
-                                              GList                   *file_list,
-                                              GArray                  *locations,
-                                              NautilusCanvasView      *canvas_view)
-{
-    g_assert (NAUTILUS_IS_CANVAS_VIEW (canvas_view));
-    g_assert (container == get_canvas_container (canvas_view));
-
-    nautilus_files_view_preview_files (NAUTILUS_FILES_VIEW (canvas_view),
-                                       file_list, locations);
-}
-
 /* this is called in one of these cases:
  * - we activate with enter holding shift
  * - we activate with space holding shift
@@ -1060,7 +1061,7 @@ canvas_container_activate_alternate_callback (NautilusCanvasContainer *container
     GdkEventButton *button_event;
     GdkEventKey *key_event;
     gboolean open_in_tab, open_in_window;
-    NautilusWindowOpenFlags flags;
+    NautilusOpenFlags flags;
 
     g_assert (NAUTILUS_IS_CANVAS_VIEW (canvas_view));
     g_assert (container == get_canvas_container (canvas_view));
@@ -1088,13 +1089,13 @@ canvas_container_activate_alternate_callback (NautilusCanvasContainer *container
 
     if (open_in_tab)
     {
-        flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
-        flags |= NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE;
+        flags |= NAUTILUS_OPEN_FLAG_NEW_TAB;
+        flags |= NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE;
     }
 
     if (open_in_window)
     {
-        flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+        flags |= NAUTILUS_OPEN_FLAG_NEW_WINDOW;
     }
 
     DEBUG ("Activate alternate, open in tab %d, new window %d\n",
@@ -1376,8 +1377,6 @@ initialize_canvas_container (NautilusCanvasView      *canvas_view,
                              G_CALLBACK (canvas_container_activate_callback), canvas_view, 0);
     g_signal_connect_object (canvas_container, "activate-alternate",
                              G_CALLBACK (canvas_container_activate_alternate_callback), canvas_view, 0);
-    g_signal_connect_object (canvas_container, "activate-previewer",
-                             G_CALLBACK (canvas_container_activate_previewer_callback), canvas_view, 0);
     g_signal_connect_object (canvas_container, "band-select-started",
                              G_CALLBACK (band_select_started_callback), canvas_view, 0);
     g_signal_connect_object (canvas_container, "band-select-ended",
@@ -1400,8 +1399,8 @@ initialize_canvas_container (NautilusCanvasView      *canvas_view,
     g_signal_connect_object (canvas_container, "get-container-uri",
                              G_CALLBACK (canvas_view_get_container_uri), canvas_view, 0);
 
-    gtk_container_add (GTK_CONTAINER (content_widget),
-                       GTK_WIDGET (canvas_container));
+    gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (content_widget),
+                                   GTK_WIDGET (canvas_container));
 
     nautilus_canvas_view_update_click_mode (canvas_view);
     nautilus_canvas_container_set_zoom_level (canvas_container,
