@@ -54,13 +54,14 @@ struct NautilusListViewDetails {
   int drag_y;
 
   gboolean drag_started;
-  gboolean ignore_button_release;
   gboolean row_selected_on_button_down;
   gboolean active;
   NautilusDragInfo *drag_source_info;
 
   GHashTable *columns;
   GtkWidget *column_editor;
+  GtkWidget *columns_popover;
+  GtkWidget *columns_popover_box;
 
   char *original_name;
 
@@ -75,5 +76,8 @@ struct NautilusListViewDetails {
 
   GtkGesture *tree_view_drag_gesture;
   GtkGesture *tree_view_multi_press_gesture;
+  GtkEventController *motion_controller;
+  GtkEventController *key_controller;
+  GtkGesture *long_press_gesture;
 };
 
