@@ -46,7 +46,6 @@
 
 typedef enum
 {
-    ACTIVATION_ACTION_ASK,
     ACTIVATION_ACTION_LAUNCH,
     ACTIVATION_ACTION_LAUNCH_IN_TERMINAL,
     ACTIVATION_ACTION_OPEN_IN_VIEW,
@@ -61,6 +60,12 @@ typedef struct
     char *uri;
 } LaunchLocation;
 
+typedef struct
+{
+    GAppInfo *application;
+    GList *uris;
+} ApplicationLaunchParameters;
+
 typedef struct
 {
     NautilusWindowSlot *slot;
@@ -71,21 +76,26 @@ typedef struct
     GList *mountables;
     GList *start_mountables;
     GList *not_mounted;
-    NautilusWindowOpenFlags flags;
+    NautilusOpenFlags flags;
     char *timed_wait_prompt;
     gboolean timed_wait_active;
     NautilusFileListHandle *files_handle;
     gboolean tried_mounting;
     char *activation_directory;
     gboolean user_confirmation;
+    GQueue *open_in_view_files;
+    GQueue *open_in_app_uris;
+    GQueue *launch_files;
+    GQueue *launch_in_terminal_files;
+    GList *open_in_app_parameters;
+    GList *unhandled_open_in_app_uris;
 } ActivateParameters;
 
 typedef struct
 {
     ActivateParameters *activation_params;
     GQueue *uris;
-    GQueue *unhandled_uris;
-} ApplicationLaunchParameters;
+} ApplicationLaunchAsyncParameters;
 
 /* Microsoft mime types at https://blogs.msdn.microsoft.com/vsofficedeveloper/2008/05/08/office-2007-file-format-mime-types-for-http-content-streaming-2/ */
 struct
@@ -221,10 +231,6 @@ struct
 /* Number of seconds until cancel dialog shows up */
 #define DELAY_UNTIL_CANCEL_MSECS 5000
 
-#define RESPONSE_RUN 1000
-#define RESPONSE_DISPLAY 1001
-#define RESPONSE_RUN_IN_TERMINAL 1002
-
 #define SILENT_WINDOW_OPEN_LIMIT 5
 #define SILENT_OPEN_LIMIT 5
 
@@ -235,6 +241,7 @@ struct
  */
 #define MAX_URI_IN_DIALOG_LENGTH 60
 
+static void activate_files_internal (ActivateParameters *parameters);
 static void cancel_activate_callback (gpointer callback_data);
 static void activate_activation_uris_ready_callback (GList   *files,
                                                      gpointer callback_data);
@@ -244,6 +251,20 @@ static void activate_callback (GList   *files,
                                gpointer callback_data);
 static void activation_mount_not_mounted (ActivateParameters *parameters);
 
+static gboolean
+is_sandboxed (void)
+{
+    static gboolean ret;
+
+    static gsize init = 0;
+    if (g_once_init_enter (&init))
+    {
+        ret = g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS);
+        g_once_init_leave (&init, 1);
+    }
+
+    return ret;
+}
 
 static void
 launch_location_free (LaunchLocation *location)
@@ -345,19 +366,27 @@ launch_locations_from_file_list (GList *list)
 }
 
 static ApplicationLaunchParameters *
-application_launch_parameters_new (ActivateParameters *activation_params,
-                                   GQueue             *uris)
+application_launch_parameters_new (GAppInfo *application,
+                                   GList    *uris)
 {
     ApplicationLaunchParameters *result;
 
     result = g_new0 (ApplicationLaunchParameters, 1);
-    result->activation_params = activation_params;
-    result->uris = uris;
-    result->unhandled_uris = g_queue_new ();
+    result->application = g_object_ref (application);
+    result->uris = g_list_copy_deep (uris, (GCopyFunc) g_strdup, NULL);
 
     return result;
 }
 
+static void
+application_launch_parameters_free (ApplicationLaunchParameters *parameters)
+{
+    g_object_unref (parameters->application);
+    g_list_free_full (parameters->uris, g_free);
+
+    g_free (parameters);
+}
+
 static gboolean
 nautilus_mime_actions_check_if_required_attributes_ready (NautilusFile *file)
 {
@@ -390,7 +419,7 @@ nautilus_mime_get_default_application_for_file (NautilusFile *file)
 
     mime_type = nautilus_file_get_mime_type (file);
     app = g_app_info_get_default_for_type (mime_type,
-                                           !nautilus_file_is_local_or_fuse (file));
+                                           !nautilus_file_has_local_path (file));
     g_free (mime_type);
 
     if (app == NULL)
@@ -519,6 +548,32 @@ trash_or_delete_files (GtkWindow   *parent_window,
     g_list_free_full (locations, g_object_unref);
 }
 
+typedef struct
+{
+    GtkWindow *parent_window;
+    NautilusFile *file;
+} TrashBrokenSymbolicLinkData;
+
+static void
+trash_symbolic_link_cb (GtkDialog *dialog,
+                        gint       response_id,
+                        gpointer   user_data)
+{
+    g_autofree TrashBrokenSymbolicLinkData *data = NULL;
+    GList file_as_list;
+
+    data = user_data;
+    gtk_widget_destroy (GTK_WIDGET (dialog));
+
+    if (response_id == GTK_RESPONSE_YES)
+    {
+        file_as_list.data = data->file;
+        file_as_list.next = NULL;
+        file_as_list.prev = NULL;
+        trash_or_delete_files (data->parent_window, &file_as_list, TRUE);
+    }
+}
+
 static void
 report_broken_symbolic_link (GtkWindow    *parent_window,
                              NautilusFile *file)
@@ -528,8 +583,8 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
     char *prompt;
     char *detail;
     GtkDialog *dialog;
-    GList file_as_list;
-    int response;
+    TrashBrokenSymbolicLinkData *data;
+
     gboolean can_trash;
 
     g_assert (nautilus_file_is_broken_symbolic_link (file));
@@ -560,15 +615,15 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
 
     if (!can_trash)
     {
-        eel_run_simple_dialog (GTK_WIDGET (parent_window), FALSE, GTK_MESSAGE_WARNING,
-                               prompt, detail, _("_Cancel"), NULL);
+        dialog = eel_show_simple_dialog (GTK_WIDGET (parent_window), GTK_MESSAGE_WARNING,
+                                         prompt, detail, _("_Cancel"), NULL);
         goto out;
     }
 
     dialog = eel_show_yes_no_dialog (prompt, detail, _("Mo_ve to Trash"), _("_Cancel"),
                                      parent_window);
 
-    gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
     /* Make this modal to avoid problems with reffing the view & file
      * to keep them around in case the view changes, which would then
@@ -579,16 +634,14 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
      * to change wildly, I don't want to mess with this now.
      */
 
-    response = gtk_dialog_run (dialog);
-    gtk_widget_destroy (GTK_WIDGET (dialog));
+    data = g_new0 (TrashBrokenSymbolicLinkData, 1);
+    data->parent_window = parent_window;
+    data->file = file;
 
-    if (response == GTK_RESPONSE_YES)
-    {
-        file_as_list.data = file;
-        file_as_list.next = NULL;
-        file_as_list.prev = NULL;
-        trash_or_delete_files (parent_window, &file_as_list, TRUE);
-    }
+    g_signal_connect (G_OBJECT (dialog),
+                      "response",
+                      G_CALLBACK (trash_symbolic_link_cb),
+                      data);
 
 out:
     g_free (prompt);
@@ -596,113 +649,10 @@ out:
     g_free (detail);
 }
 
-static ActivationAction
-get_executable_text_file_action (GtkWindow    *parent_window,
-                                 NautilusFile *file)
-{
-    GtkDialog *dialog;
-    char *file_name;
-    char *prompt;
-    char *detail;
-    int preferences_value;
-    int response;
-
-    g_assert (nautilus_file_contains_text (file));
-
-    preferences_value = g_settings_get_enum (nautilus_preferences,
-                                             NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION);
-    switch (preferences_value)
-    {
-        case NAUTILUS_EXECUTABLE_TEXT_LAUNCH:
-        {
-            return ACTIVATION_ACTION_LAUNCH;
-        }
-
-        case NAUTILUS_EXECUTABLE_TEXT_DISPLAY:
-        {
-            return ACTIVATION_ACTION_OPEN_IN_APPLICATION;
-        }
-
-        case NAUTILUS_EXECUTABLE_TEXT_ASK:
-        {
-        }
-        break;
-
-        default:
-            /* Complain non-fatally, since preference data can't be trusted */
-            g_warning ("Unknown value %d for NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION",
-                       preferences_value);
-    }
-
-
-    file_name = nautilus_file_get_display_name (file);
-    prompt = g_strdup_printf (_("Do you want to run “%s”, or display its contents?"),
-                              file_name);
-    detail = g_strdup_printf (_("“%s” is an executable text file."),
-                              file_name);
-    g_free (file_name);
-
-    dialog = eel_create_question_dialog (prompt,
-                                         detail,
-                                         _("Run in _Terminal"), RESPONSE_RUN_IN_TERMINAL,
-                                         _("_Display"), RESPONSE_DISPLAY,
-                                         parent_window);
-    gtk_dialog_add_button (dialog, _("_Cancel"), GTK_RESPONSE_CANCEL);
-    gtk_dialog_add_button (dialog, _("_Run"), RESPONSE_RUN);
-    gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
-    gtk_widget_show (GTK_WIDGET (dialog));
-
-    g_free (prompt);
-    g_free (detail);
-
-    response = gtk_dialog_run (dialog);
-    gtk_widget_destroy (GTK_WIDGET (dialog));
-
-    switch (response)
-    {
-        case RESPONSE_RUN:
-        {
-            return ACTIVATION_ACTION_LAUNCH;
-        }
-
-        case RESPONSE_RUN_IN_TERMINAL:
-        {
-            return ACTIVATION_ACTION_LAUNCH_IN_TERMINAL;
-        }
-
-        case RESPONSE_DISPLAY:
-        {
-            return ACTIVATION_ACTION_OPEN_IN_APPLICATION;
-        }
-
-        default:
-            return ACTIVATION_ACTION_DO_NOTHING;
-    }
-}
-
 static ActivationAction
 get_default_executable_text_file_action (void)
 {
-    int preferences_value;
-
-    preferences_value = g_settings_get_enum (nautilus_preferences,
-                                             NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION);
-    switch (preferences_value)
-    {
-        case NAUTILUS_EXECUTABLE_TEXT_LAUNCH:
-        {
-            return ACTIVATION_ACTION_LAUNCH;
-        }
-
-        case NAUTILUS_EXECUTABLE_TEXT_DISPLAY:
-        {
-            return ACTIVATION_ACTION_OPEN_IN_APPLICATION;
-        }
-
-        case NAUTILUS_EXECUTABLE_TEXT_ASK:
-        default:
-            return ACTIVATION_ACTION_ASK;
-    }
+    return ACTIVATION_ACTION_OPEN_IN_APPLICATION;
 }
 
 static ActivationAction
@@ -792,6 +742,114 @@ nautilus_mime_file_opens_in_external_app (NautilusFile *file)
     return (activation_action == ACTIVATION_ACTION_OPEN_IN_APPLICATION);
 }
 
+
+static unsigned int
+mime_application_hash (GAppInfo *app)
+{
+    const char *id;
+
+    id = g_app_info_get_id (app);
+
+    if (id == NULL)
+    {
+        return GPOINTER_TO_UINT (app);
+    }
+
+    return g_str_hash (id);
+}
+
+static void
+list_to_parameters_foreach (GAppInfo  *application,
+                            GList     *uris,
+                            GList    **ret)
+{
+    ApplicationLaunchParameters *parameters;
+
+    uris = g_list_reverse (uris);
+
+    parameters = application_launch_parameters_new
+                     (application, uris);
+    *ret = g_list_prepend (*ret, parameters);
+}
+
+
+/**
+ * make_activation_parameters
+ *
+ * Construct a list of ApplicationLaunchParameters from a list of NautilusFiles,
+ * where files that have the same default application are put into the same
+ * launch parameter, and others are put into the unhandled_files list.
+ *
+ * @files: Files to use for construction.
+ * @unhandled_files: Files without any default application will be put here.
+ *
+ * Return value: Newly allocated list of ApplicationLaunchParameters.
+ **/
+static GList *
+make_activation_parameters (GList  *uris,
+                            GList **unhandled_uris)
+{
+    GList *ret, *l, *app_uris;
+    NautilusFile *file;
+    GAppInfo *app, *old_app;
+    GHashTable *app_table;
+    char *uri;
+
+    ret = NULL;
+    *unhandled_uris = NULL;
+
+    app_table = g_hash_table_new_full
+                    ((GHashFunc) mime_application_hash,
+                    (GEqualFunc) g_app_info_equal,
+                    (GDestroyNotify) g_object_unref,
+                    (GDestroyNotify) g_list_free);
+
+    for (l = uris; l != NULL; l = l->next)
+    {
+        uri = l->data;
+        file = nautilus_file_get_by_uri (uri);
+
+        app = nautilus_mime_get_default_application_for_file (file);
+        if (app != NULL)
+        {
+            app_uris = NULL;
+
+            if (g_hash_table_lookup_extended (app_table, app,
+                                              (gpointer *) &old_app,
+                                              (gpointer *) &app_uris))
+            {
+                g_hash_table_steal (app_table, old_app);
+
+                app_uris = g_list_prepend (app_uris, uri);
+
+                g_object_unref (app);
+                app = old_app;
+            }
+            else
+            {
+                app_uris = g_list_prepend (NULL, uri);
+            }
+
+            g_hash_table_insert (app_table, app, app_uris);
+        }
+        else
+        {
+            *unhandled_uris = g_list_prepend (*unhandled_uris, uri);
+        }
+        nautilus_file_unref (file);
+    }
+
+    g_hash_table_foreach (app_table,
+                          (GHFunc) list_to_parameters_foreach,
+                          &ret);
+
+    g_hash_table_destroy (app_table);
+
+    *unhandled_uris = g_list_reverse (*unhandled_uris);
+
+    return g_list_reverse (ret);
+}
+
 static gboolean
 file_was_cancelled (NautilusFile *file)
 {
@@ -840,13 +898,18 @@ activation_parameters_free (ActivateParameters *parameters)
     g_free (parameters->activation_directory);
     g_free (parameters->timed_wait_prompt);
     g_assert (parameters->files_handle == NULL);
+    g_clear_pointer (&parameters->open_in_view_files, g_queue_free);
+    g_clear_pointer (&parameters->open_in_app_uris, g_queue_free);
+    g_clear_pointer (&parameters->launch_files, g_queue_free);
+    g_clear_pointer (&parameters->launch_in_terminal_files, g_queue_free);
+    g_list_free (parameters->open_in_app_parameters);
+    g_list_free (parameters->unhandled_open_in_app_uris);
     g_free (parameters);
 }
 
 static void
-application_launch_parameters_free (ApplicationLaunchParameters *parameters)
+application_launch_async_parameters_free (ApplicationLaunchAsyncParameters *parameters)
 {
-    g_queue_free (parameters->unhandled_uris);
     g_queue_free (parameters->uris);
     activation_parameters_free (parameters->activation_params);
 
@@ -921,41 +984,56 @@ activate_mount_op_active (GtkMountOperation  *operation,
     }
 }
 
-static gboolean
-confirm_multiple_windows (GtkWindow *parent_window,
-                          int        count,
-                          gboolean   use_tabs)
+static GtkDialog *
+show_confirm_multiple (GtkWindow *parent_window,
+                       int        window_count,
+                       int        tab_count)
 {
     GtkDialog *dialog;
     char *prompt;
     char *detail;
-    int response;
-
-    if (count <= SILENT_WINDOW_OPEN_LIMIT)
-    {
-        return TRUE;
-    }
 
     prompt = _("Are you sure you want to open all files?");
-    if (use_tabs)
+    if (tab_count > 0 && window_count > 0)
+    {
+        int count = tab_count + window_count;
+        detail = g_strdup_printf (ngettext ("This will open %d separate tab and window.",
+                                            "This will open %d separate tabs and windows.", count), count);
+    }
+    else if (tab_count > 0)
     {
         detail = g_strdup_printf (ngettext ("This will open %d separate tab.",
-                                            "This will open %d separate tabs.", count), count);
+                                            "This will open %d separate tabs.", tab_count), tab_count);
     }
     else
     {
         detail = g_strdup_printf (ngettext ("This will open %d separate window.",
-                                            "This will open %d separate windows.", count), count);
+                                            "This will open %d separate windows.", window_count), window_count);
     }
     dialog = eel_show_yes_no_dialog (prompt, detail,
                                      _("_OK"), _("_Cancel"),
                                      parent_window);
     g_free (detail);
 
-    response = gtk_dialog_run (dialog);
+    return dialog;
+}
+
+static void
+on_confirm_multiple_windows_response (GtkDialog          *dialog,
+                                      int                 response_id,
+                                      ActivateParameters *parameters)
+{
     gtk_widget_destroy (GTK_WIDGET (dialog));
 
-    return response == GTK_RESPONSE_YES;
+    if (response_id == GTK_RESPONSE_YES)
+    {
+        unpause_activation_timed_cancel (parameters);
+        activate_files_internal (parameters);
+    }
+    else
+    {
+        activation_parameters_free (parameters);
+    }
 }
 
 typedef struct
@@ -964,7 +1042,7 @@ typedef struct
     GtkWindow *parent_window;
     NautilusFile *file;
     GList *files;
-    NautilusWindowOpenFlags flags;
+    NautilusOpenFlags flags;
     char *activation_directory;
     gboolean user_confirmation;
     char *uri;
@@ -1370,25 +1448,23 @@ out:
 }
 
 static void
-on_launch_default_for_uri (GObject      *source_object,
-                           GAsyncResult *res,
-                           gpointer      user_data)
+launch_default_for_uris_callback (GObject      *source_object,
+                                  GAsyncResult *res,
+                                  gpointer      user_data)
 {
-    ApplicationLaunchParameters *params;
+    ApplicationLaunchAsyncParameters *params;
     ActivateParameters *activation_params;
     char *uri;
-    gboolean sandboxed;
-    GError *error = NULL;
+    g_autoptr (GError) error = NULL;
 
     params = user_data;
     activation_params = params->activation_params;
     uri = g_queue_pop_head (params->uris);
-    sandboxed = g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS);
 
     nautilus_launch_default_for_uri_finish (res, &error);
-    if (!sandboxed && error != NULL && error->code != G_IO_ERROR_CANCELLED)
+    if (error == NULL)
     {
-        g_queue_push_tail (params->unhandled_uris, uri);
+        gtk_recent_manager_add_item (gtk_recent_manager_get_default (), uri);
     }
 
     if (!g_queue_is_empty (params->uris))
@@ -1396,17 +1472,12 @@ on_launch_default_for_uri (GObject      *source_object,
         nautilus_launch_default_for_uri_async (g_queue_peek_head (params->uris),
                                                activation_params->parent_window,
                                                activation_params->cancellable,
-                                               on_launch_default_for_uri,
+                                               launch_default_for_uris_callback,
                                                params);
     }
     else
     {
-        while ((uri = g_queue_pop_head (params->unhandled_uris)) != NULL)
-        {
-            application_unhandled_uri (activation_params, uri);
-        }
-
-        application_launch_parameters_free (params);
+        application_launch_async_parameters_free (params);
     }
 }
 
@@ -1414,21 +1485,16 @@ static void
 activate_files (ActivateParameters *parameters)
 {
     NautilusFile *file;
-    NautilusWindowOpenFlags flags;
     int count;
-    g_autofree char *old_working_dir = NULL;
-    GdkScreen *screen;
-    g_autoptr (GQueue) launch_files = NULL;
-    g_autoptr (GQueue) launch_in_terminal_files = NULL;
-    g_autoptr (GQueue) open_in_app_uris = NULL;
-    g_autoptr (GQueue) open_in_view_files = NULL;
+    gint num_windows = 0;
+    gint num_tabs = 0;
     GList *l;
     ActivationAction action;
 
-    launch_files = g_queue_new ();
-    launch_in_terminal_files = g_queue_new ();
-    open_in_view_files = g_queue_new ();
-    open_in_app_uris = g_queue_new ();
+    parameters->launch_files = g_queue_new ();
+    parameters->launch_in_terminal_files = g_queue_new ();
+    parameters->open_in_view_files = g_queue_new ();
+    parameters->open_in_app_uris = g_queue_new ();
 
     for (l = parameters->locations; l != NULL; l = l->next)
     {
@@ -1443,39 +1509,30 @@ activate_files (ActivateParameters *parameters)
         }
 
         action = get_activation_action (file);
-        if (action == ACTIVATION_ACTION_ASK)
-        {
-            /* Special case for executable text files, since it might be
-             * dangerous & unexpected to launch these.
-             */
-            pause_activation_timed_cancel (parameters);
-            action = get_executable_text_file_action (parameters->parent_window, file);
-            unpause_activation_timed_cancel (parameters);
-        }
 
         switch (action)
         {
             case ACTIVATION_ACTION_LAUNCH:
             {
-                g_queue_push_tail (launch_files, file);
+                g_queue_push_tail (parameters->launch_files, file);
             }
             break;
 
             case ACTIVATION_ACTION_LAUNCH_IN_TERMINAL:
             {
-                g_queue_push_tail (launch_in_terminal_files, file);
+                g_queue_push_tail (parameters->launch_in_terminal_files, file);
             }
             break;
 
             case ACTIVATION_ACTION_OPEN_IN_VIEW:
             {
-                g_queue_push_tail (open_in_view_files, file);
+                g_queue_push_tail (parameters->open_in_view_files, file);
             }
             break;
 
             case ACTIVATION_ACTION_OPEN_IN_APPLICATION:
             {
-                g_queue_push_tail (open_in_app_uris, location->uri);
+                g_queue_push_tail (parameters->open_in_app_uris, location->uri);
             }
             break;
 
@@ -1490,25 +1547,77 @@ activate_files (ActivateParameters *parameters)
                 g_assert_not_reached ();
             }
             break;
-
-            case ACTIVATION_ACTION_ASK:
-            {
-                g_assert_not_reached ();
-            }
-            break;
         }
     }
 
+    count = g_queue_get_length (parameters->open_in_view_files);
+    if (count > 1)
+    {
+        if ((parameters->flags & NAUTILUS_OPEN_FLAG_NEW_WINDOW) == 0)
+        {
+            parameters->flags |= NAUTILUS_OPEN_FLAG_NEW_TAB;
+            num_tabs += count;
+        }
+        else
+        {
+            parameters->flags |= NAUTILUS_OPEN_FLAG_NEW_WINDOW;
+            num_windows += count;
+        }
+    }
+
+    if (parameters->open_in_app_uris != NULL)
+    {
+        if (is_sandboxed ())
+        {
+            num_windows += g_queue_get_length (parameters->open_in_app_uris);
+        }
+        else
+        {
+            parameters->open_in_app_parameters = make_activation_parameters (g_queue_peek_head_link (parameters->open_in_app_uris),
+                                                                             &parameters->unhandled_open_in_app_uris);
+            num_windows += g_list_length (parameters->open_in_app_parameters);
+            num_windows += g_list_length (parameters->unhandled_open_in_app_uris);
+        }
+    }
+
+    num_windows += g_queue_get_length (parameters->launch_files);
+    num_windows += g_queue_get_length (parameters->launch_in_terminal_files);
+
+    if (parameters->user_confirmation &&
+        num_tabs + num_windows > SILENT_OPEN_LIMIT)
+    {
+        GtkDialog *dialog;
+
+        pause_activation_timed_cancel (parameters);
+        dialog = show_confirm_multiple (parameters->parent_window, num_windows, num_tabs);
+        g_signal_connect (dialog, "response",
+                          G_CALLBACK (on_confirm_multiple_windows_response), parameters);
+    }
+    else
+    {
+        activate_files_internal (parameters);
+    }
+}
+
+static void
+activate_files_internal (ActivateParameters *parameters)
+{
+    NautilusFile *file;
+    ApplicationLaunchParameters *one_parameters;
+    g_autofree char *old_working_dir = NULL;
+    GdkScreen *screen;
+    GList *l;
+
     if (parameters->activation_directory &&
-        (!g_queue_is_empty (launch_files) ||
-         !g_queue_is_empty (launch_in_terminal_files)))
+        (!g_queue_is_empty (parameters->launch_files) ||
+         !g_queue_is_empty (parameters->launch_in_terminal_files)))
     {
         old_working_dir = g_get_current_dir ();
         g_chdir (parameters->activation_directory);
     }
 
     screen = gtk_widget_get_screen (GTK_WIDGET (parameters->parent_window));
-    for (l = g_queue_peek_head_link (launch_files); l != NULL; l = l->next)
+    for (l = g_queue_peek_head_link (parameters->launch_files); l != NULL; l = l->next)
     {
         g_autofree char *uri = NULL;
         g_autofree char *executable_path = NULL;
@@ -1525,7 +1634,7 @@ activate_files (ActivateParameters *parameters)
         nautilus_launch_application_from_command (screen, quoted_path, FALSE, NULL);
     }
 
-    for (l = g_queue_peek_head_link (launch_in_terminal_files); l != NULL; l = l->next)
+    for (l = g_queue_peek_head_link (parameters->launch_in_terminal_files); l != NULL; l = l->next)
     {
         g_autofree char *uri = NULL;
         g_autofree char *executable_path = NULL;
@@ -1547,27 +1656,9 @@ activate_files (ActivateParameters *parameters)
         g_chdir (old_working_dir);
     }
 
-    count = g_queue_get_length (open_in_view_files);
-
-    flags = parameters->flags;
-    if (count > 1)
+    if (parameters->slot != NULL)
     {
-        if ((parameters->flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0)
-        {
-            flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
-        }
-        else
-        {
-            flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
-        }
-    }
-
-    if (parameters->slot != NULL &&
-        (!parameters->user_confirmation ||
-         confirm_multiple_windows (parameters->parent_window, count,
-                                   (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0)))
-    {
-        if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0 &&
+        if ((parameters->flags & NAUTILUS_OPEN_FLAG_NEW_TAB) != 0 &&
             g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION) ==
             NAUTILUS_NEW_TAB_POSITION_AFTER_CURRENT_TAB)
         {
@@ -1576,10 +1667,10 @@ activate_files (ActivateParameters *parameters)
              * Each of them is appended to the current tab, i.e.
              * prepended to the list of tabs to open.
              */
-            g_queue_reverse (open_in_view_files);
+            g_queue_reverse (parameters->open_in_view_files);
         }
 
-        for (l = g_queue_peek_head_link (open_in_view_files); l != NULL; l = l->next)
+        for (l = g_queue_peek_head_link (parameters->open_in_view_files); l != NULL; l = l->next)
         {
             g_autofree char *uri = NULL;
             g_autoptr (GFile) location = NULL;
@@ -1608,30 +1699,51 @@ activate_files (ActivateParameters *parameters)
              * in some cases. Until we figure out what's going on, continue to use the parameters->slot
              * to make splicit the window we want to use for activating the files */
             nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
-                                                     location_with_permissions, flags, NULL, NULL, parameters->slot);
+                                                     location_with_permissions, parameters->flags, NULL, NULL, parameters->slot);
         }
     }
 
-    if (g_queue_is_empty (open_in_app_uris))
-    {
-        activation_parameters_free (parameters);
-    }
-    else
+    if (!g_queue_is_empty (parameters->open_in_app_uris) && is_sandboxed ())
     {
         const char *uri;
-        ApplicationLaunchParameters *params;
+        ApplicationLaunchAsyncParameters *async_params;
 
-        uri = g_queue_peek_head (open_in_app_uris);
-        params = application_launch_parameters_new (parameters,
-                                                    g_queue_copy (open_in_app_uris));
+        uri = g_queue_peek_head (parameters->open_in_app_uris);
+
+        async_params = g_new0 (ApplicationLaunchAsyncParameters, 1);
+        async_params->activation_params = parameters;
+        async_params->uris = g_steal_pointer (&parameters->open_in_app_uris);
 
-        gtk_recent_manager_add_item (gtk_recent_manager_get_default (), uri);
         nautilus_launch_default_for_uri_async (uri,
                                                parameters->parent_window,
                                                parameters->cancellable,
-                                               on_launch_default_for_uri,
-                                               params);
+                                               launch_default_for_uris_callback,
+                                               async_params);
+        return;
     }
+
+    if (!g_queue_is_empty (parameters->open_in_app_uris))
+    {
+        for (l = parameters->open_in_app_parameters; l != NULL; l = l->next)
+        {
+            one_parameters = l->data;
+
+            nautilus_launch_application_by_uri (one_parameters->application,
+                                                one_parameters->uris,
+                                                parameters->parent_window);
+            application_launch_parameters_free (one_parameters);
+        }
+
+        for (l = parameters->unhandled_open_in_app_uris; l != NULL; l = l->next)
+        {
+            char *uri = l->data;
+
+            /* this does not block */
+            application_unhandled_uri (parameters, uri);
+        }
+    }
+
+    activation_parameters_free (parameters);
 }
 
 static void
@@ -2098,12 +2210,12 @@ activation_start_mountables (ActivateParameters *parameters)
  *
  **/
 void
-nautilus_mime_activate_files (GtkWindow               *parent_window,
-                              NautilusWindowSlot      *slot,
-                              GList                   *files,
-                              const char              *launch_directory,
-                              NautilusWindowOpenFlags  flags,
-                              gboolean                 user_confirmation)
+nautilus_mime_activate_files (GtkWindow          *parent_window,
+                              NautilusWindowSlot *slot,
+                              GList              *files,
+                              const char         *launch_directory,
+                              NautilusOpenFlags   flags,
+                              gboolean            user_confirmation)
 {
     ActivateParameters *parameters;
     char *file_name;
@@ -2195,11 +2307,11 @@ nautilus_mime_activate_files (GtkWindow               *parent_window,
  **/
 
 void
-nautilus_mime_activate_file (GtkWindow               *parent_window,
-                             NautilusWindowSlot      *slot,
-                             NautilusFile            *file,
-                             const char              *launch_directory,
-                             NautilusWindowOpenFlags  flags)
+nautilus_mime_activate_file (GtkWindow          *parent_window,
+                             NautilusWindowSlot *slot,
+                             NautilusFile       *file,
+                             const char         *launch_directory,
+                             NautilusOpenFlags   flags)
 {
     GList *files;
 
